package ch.zhaw.gartenverwaltung.io;

import ch.zhaw.gartenverwaltung.types.Plant;
import ch.zhaw.gartenverwaltung.types.Task;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateDeserializer;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.time.LocalDate;
import java.time.MonthDay;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class JsonTaskDatabase implements TaskDatabase{
    private final URL dataSource = getClass().getResource("taskdb.json");

    private Map<Long, Task> taskMap = Collections.emptyMap();



    private final static JavaTimeModule timeModule = new JavaTimeModule();
    static {
        DateTimeFormatter dateFormat = DateTimeFormatter.ofPattern("yy-MM-dd");
        LocalDateDeserializer dateDeserializer = new LocalDateDeserializer(dateFormat);
        timeModule.addDeserializer(LocalDate.class, dateDeserializer);
    }

    @Override
    public List<Task> getTaskList(LocalDate start, LocalDate end) throws IOException{
        List<Task> taskList = Collections.emptyList();

        if (dataSource != null) {
            ObjectMapper mapper = new ObjectMapper();
            mapper.registerModule(timeModule);

            taskList = mapper.readerForListOf(Task.class).readValue(dataSource);
        }
        return taskList;
    }

    @Override
    public void saveTask(Task task) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        if(dataSource != null) {
            mapper.writeValue(new File(dataSource.getFile()), task);
        }
    }

    @Override
    public void removeTask(Task task) throws IOException {
        List<Task> taskList;

        if(dataSource != null) {
            ObjectMapper mapper = new ObjectMapper();
            taskList = mapper.readerForListOf(Task.class).readValue(dataSource);

            for (int index = 0; index < taskList.size(); index++){
                if(task.equals(taskList.get(index))){
                    taskList.remove(task);
                    break;
                }
            }
        }
    }

    private void writeTaskListToFile() throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        mapper.registerModule(timeModule);

        mapper.writeValue(new File(dataSource.getFile()), taskMap);
    }

    private void loadTaskListFromFile() throws IOException {

        ObjectMapper mapper = new ObjectMapper();
        mapper.registerModule(timeModule);

        List<Task> result;
        result = mapper.readerForListOf(Task.class).readValue(dataSource);

        //taskMap = result.stream().collect(HashMap::new,
    }

    public static void main(String[] args) {
        LocalDate date = LocalDate.now();
        LocalDate yesterday = LocalDate.now();
        yesterday.minusDays(1);

        Task testTask = new Task("water", "apply water", date);
        JsonTaskDatabase jsonTaskDatabase = new JsonTaskDatabase();
        try {
            jsonTaskDatabase.saveTask(testTask);
            System.out.println(jsonTaskDatabase.getTaskList(yesterday,date));
        } catch (Exception e){
            System.out.println("Task saving failed!");
        }
    }
}
